unit SMSSender;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, NokConn, NokSMS, NokUtils;

type
  TSMSMsgEvent = procedure (Index:integer;Time:TDateTime;Text,Send:String) of object;
  TSMSRepEvent = procedure (Index:integer;Time:TDateTime;Send:String;Delivered,Delivery:integer) of object;

  TOxygenSMS = class(TCustomControl)
  private
   nc:TNokiaConnection;
   sms:TNokiaSMS;
   smsc:TSMSCenterProfile;
   fMsgReceived:TSMSMsgEvent;
   fRepReceived:TSMSRepEvent;
   function  GetComNumber:integer;
   procedure SetComNumber(Value:integer);
   function  GetState:integer;
   procedure MsgRec(MsgInfo:TMessageInfo);
   procedure RepRec(MsgInfo:TMessageInfo);
  protected
   procedure WMPaint(var Msg:TWMPaint);message WM_Paint;
   procedure Loaded;override;
  public
   constructor Create(AOwner:TComponent); override;
   destructor Destroy; override;
   function  Open:Boolean;
   procedure Close;
   function  GetSMSCNumber:string;
   procedure SetSMSCNumber(PhoneNumber:string);
   function  SendSMSMessage(PhoneNumber:string;Text:string):Boolean;
   function  GetInboxCount:integer;
   function  ReadSMSMessage(Index:Longint; var IsMsg:WordBool; var Time:TDateTime; var Text,Send:String;var Delivered,Delivery:integer):Boolean;
   function  DeleteSMSMessage(Index:integer):Boolean;
  published
   property ComNumber:integer Read GetComNumber Write SetComNumber;
   property State:integer Read GetState;
   property OnSMSMessageReceived:TSMSMsgEvent Read fMsgReceived Write fMsgReceived;
   property OnSMSReportReceived:TSMSRepEvent Read fRepReceived Write fRepReceived;
  end;

procedure Register;

implementation

{$R SMSSender.dcr}

uses ShellAPI;

constructor TOxygenSMS.Create(AOwner:TComponent);
begin
 inherited;
 Visible:=False;
 TabStop:=False;
 Height:=32;
 Width:=32;
 nc:=TNokiaConnection.Create(Self);
 sms:=TNokiaSMS.Create(Self);
 sms.Connection:=nc;
 sms.OnSMSMessageReceived:=MsgRec;
 sms.OnSMSReportReceived:=RepRec;
end;

destructor TOxygenSMS.Destroy;
begin
 sms.Destroy;
 nc.Destroy;
 inherited;
end;

function TOxygenSMS.Open:Boolean;
begin
 Result:=nc.Open;
 If Result then GetSMSCNumber;
end;

procedure TOxygenSMS.Close;
begin
 nc.Close;
end;

function TOxygenSMS.SendSMSMessage(PhoneNumber:string;Text:string):Boolean;
 var Msg:TMessageInfo;
begin
 Msg.Text:=Text;
 Msg.Validity:=smsc.Validity;
 Msg.SMSCenter.Number:=smsc.Number.Number;
 Msg.SMSCenter.Domestic:=smsc.Number.Domestic;
 If (Length(PhoneNumber)>0) and (PhoneNumber[1]='+') then begin
  Msg.Destination.Number:=Copy(PhoneNumber,2,Length(PhoneNumber)-1);
  Msg.Destination.Domestic:=False;
 end
 else begin
  Msg.Destination.Number:=PhoneNumber;
  Msg.Destination.Domestic:=True;
 end;
 Result:=sms.SendMessage(Msg,False);
end;

function TOxygenSMS.GetInboxCount:integer;
begin
 Result:=sms.TotalMessages;
end;

function TOxygenSMS.ReadSMSMessage(Index:Longint;var IsMsg:WordBool; var Time:TDateTime; var Text,Send:String;var Delivered,Delivery:integer):Boolean;
 var Msg:TMessageInfo;
begin
 Msg:=sms.ReadMessage(Index);
 Result:= LastError=perNone;
 If Index=0 then Result:=False
 else begin
  IsMsg:=Msg.Kind=mkMT;
  Time:=Msg.Time;
  Text:=Msg.Text;
  Send:=Msg.Sender.Number;
  If not Msg.Sender.Domestic then Send:='+'+Send;
  Delivered:=Ord(Msg.Delivered);
  Delivery:=Ord(Msg.Delivery);
 end;
end;

function TOxygenSMS.DeleteSMSMessage(Index:integer):Boolean;
begin
 Result:=sms.DeleteMessage(Index);
end;

///////////////////////////////////////////////////////////////////////////////////////

procedure TOxygenSMS.Loaded;
begin
 inherited;
 Visible:=False;
 Width:=32;
 Height:=32;
end;

procedure TOxygenSMS.WMPaint(var Msg:TWMPaint);
 var b:TBitmap;
begin
 inherited;
 Width:=32;
 Height:=32;
 b:=TBitmap.Create;
 b.LoadFromResourceName(HInstance,'TOxygenSMS');
 Canvas.Draw(1,1,b);
 b.Free;
end;

function TOxygenSMS.GetComNumber:integer;
begin
 Result:=nc.ComNumber;
end;

procedure TOxygenSMS.SetComNumber(Value:integer);
begin
 nc.ComNumber:=Value;
end;

function TOxygenSMS.GetState:integer;
begin
 Case nc.State of
  stClosed:Result:=0;
  stOpen:Result:=1;
  stBusy:Result:=2;
  else Result:=0;
 End;
end;

function TOxygenSMS.GetSMSCNumber:string;
begin
 smsc:=sms.SMSCenter[1];
 Result:=smsc.Number.Number;
 If not smsc.Number.Domestic then Result:='+'+Result;
end;

procedure TOxygenSMS.SetSMSCNumber(PhoneNumber:string);
begin
 If (Length(PhoneNumber)>0) and (PhoneNumber[1]='+') then begin
  smsc.Number.Number:=Copy(PhoneNumber,2,Length(PhoneNumber)-1);
  smsc.Number.Domestic:=False;
 end
 else begin
  smsc.Number.Number:=PhoneNumber;
  smsc.Number.Domestic:=True;
 end;
 sms.SMSCenter[1]:=smsc;
end;

procedure TOxygenSMS.MsgRec(MsgInfo:TMessageInfo);
begin
 If Assigned(fMsgReceived) then
  If MsgInfo.Sender.Domestic
   then fMsgReceived(MsgInfo.Number,MsgInfo.Time,MsgInfo.Text,MsgInfo.Sender.Number)
   else fMsgReceived(MsgInfo.Number,MsgInfo.Time,MsgInfo.Text,'+'+MsgInfo.Sender.Number);
end;

procedure TOxygenSMS.RepRec(MsgInfo:TMessageInfo);
begin
 If Assigned(fRepReceived) then
  If MsgInfo.Sender.Domestic
   then fRepReceived(MsgInfo.Number,MsgInfo.Time,MsgInfo.Sender.Number,Ord(MsgInfo.Delivered),Ord(MsgInfo.Delivery))
   else fRepReceived(MsgInfo.Number,MsgInfo.Time,'+'+MsgInfo.Sender.Number,Ord(MsgInfo.Delivered),Ord(MsgInfo.Delivery));
end;

////////////////////////////////////////////////////////////////////////////////////

procedure Register;
begin
  RegisterComponents('Oxygen', [TOxygenSMS]);
end;

end.
